/* Styles for admin pages */

/* General */

.truncate {
    display: inline-block;
    width: 20em;
    white-space: nowrap;
    overflow: hidden !important;
    text-overflow: ellipsis;
}

.text-danger {
    color: var(--error-fg)
}

.text-warning {
    color: var(--accent)
}

.text-muted {
    color: var(--close-button-hover-bg)
}

.state-retry {
    color: var(--secondary)
}

.field-_runtime {
    text-align: right;
}

.text-right {
    text-align: right;
}

/* result list */

#result_list {
    width: 100%;
}
