#result_list thead th:nth-child(2),
#result_list thead th:nth-child(3),
#result_list thead th:nth-child(4),
#result_list thead th:nth-child(5),
#result_list thead th:nth-child(6),
#result_list thead th:nth-child(7),
#result_list thead th:nth-child(8),
#result_list thead th:nth-child(9) {
    text-align: right;
}

#result_list tbody td:nth-child(2),
#result_list tbody td:nth-child(3),
#result_list tbody td:nth-child(4),
#result_list tbody td:nth-child(5),
#result_list tbody td:nth-child(6),
#result_list tbody td:nth-child(7),
#result_list tbody td:nth-child(8),
#result_list tbody td:nth-child(9) {
    text-align: right;
}
