{% extends 'admin/change_list.html' %}
{% load i18n humanize static %}

{% block breadcrumbs %}
    <div class="breadcrumbs">
        <a href="{% url 'admin:index' %}">{% translate 'Home' %}</a>
        &rsaquo; <a href="{% url 'admin:app_list' app_label=cl.opts.app_label %}">{{ cl.opts.app_config.verbose_name }}</a>
        &rsaquo; {{ title }}
    </div>
{% endblock %}

{% block content %}
    <p>Report rendered: {{ report_name }}</p>
{% endblock %}
