# Generated by Django 4.0.7 on 2022-09-26 20:47

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("taskmonitor", "0002_taskreport"),
    ]

    operations = [
        migrations.CreateModel(
            name="QueuedTask",
            fields=[
                ("app_name", models.CharField(max_length=255)),
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4, primary_key=True, serialize=False
                    ),
                ),
                ("name", models.CharField(max_length=255)),
                ("priority", models.PositiveIntegerField(default=None, null=True)),
                ("position", models.PositiveIntegerField()),
            ],
            options={
                "managed": False,
            },
        ),
    ]
