{% load static %}

<div id="{{ data_id }}-container" style="width: 100%; height: 550px;">
    {% if NIGHT_MODE %}
        <img src="{% static 'taskmonitor/images/Spinner-1s-64px-dark.gif' %}"/>
    {% else %}
        <img src="{% static 'taskmonitor/images/Spinner-1s-64px-light.gif' %}"/>
    {% endif %}
</div>
