# Generated by Django 4.0.10 on 2023-07-04 13:09

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("taskmonitor", "0006_truncate_exceptions"),
    ]

    operations = [
        migrations.AlterField(
            model_name="tasklog",
            name="exception",
            field=models.CharField(
                db_index=True,
                help_text="Name of the raised exception if any.",
                max_length=255,
            ),
        ),
    ]
